@@ -1,7 +1,7 @@
 /*
  * MessagePack packing routine template
  *
- * Copyright (C) 2008-2009 FURUHASHI Sadayuki
+ * Copyright (C) 2008-2010 FURUHASHI Sadayuki
  *
  *    Licensed under the Apache License, Version 2.0 (the "License");
  *    you may not use this file except in compliance with the License.
@@ -16,88 +16,16 @@
  *    limitations under the License.
  */
 
-#if !defined(__LITTLE_ENDIAN__) && !defined(__BIG_ENDIAN__)
-#if __BYTE_ORDER == __LITTLE_ENDIAN
-#define __LITTLE_ENDIAN__
-#elif __BYTE_ORDER == __BIG_ENDIAN
-#define __BIG_ENDIAN__
-#endif
-#endif
-
-
 #ifdef __LITTLE_ENDIAN__
-
-#define STORE8_BE8(d) \
-	((uint8_t*)&d)[0]
-
-
-#define STORE16_BE8(d) \
-	((uint8_t*)&d)[0]
-
-#define STORE16_BE16(d) \
-	((uint8_t*)&d)[1], ((uint8_t*)&d)[0]
-
-
-#define STORE32_BE8(d) \
-	((uint8_t*)&d)[0]
-
-#define STORE32_BE16(d) \
-	((uint8_t*)&d)[1], ((uint8_t*)&d)[0]
-
-#define STORE32_BE32(d) \
-	((uint8_t*)&d)[3], ((uint8_t*)&d)[2], ((uint8_t*)&d)[1], ((uint8_t*)&d)[0]
-
-
-#define STORE64_BE8(d) \
-	((uint8_t*)&d)[0]
-
-#define STORE64_BE16(d) \
-	((uint8_t*)&d)[1], ((uint8_t*)&d)[0]
-
-#define STORE64_BE32(d) \
-	((uint8_t*)&d)[3], ((uint8_t*)&d)[2], ((uint8_t*)&d)[1], ((uint8_t*)&d)[0]
-
-#define STORE64_BE64(d) \
-	((uint8_t*)&d)[7], ((uint8_t*)&d)[6], ((uint8_t*)&d)[5], ((uint8_t*)&d)[4], \
-	((uint8_t*)&d)[3], ((uint8_t*)&d)[2], ((uint8_t*)&d)[1], ((uint8_t*)&d)[0]
-
-
+#define TAKE8_8(d)  ((uint8_t*)&d)[0]
+#define TAKE8_16(d) ((uint8_t*)&d)[0]
+#define TAKE8_32(d) ((uint8_t*)&d)[0]
+#define TAKE8_64(d) ((uint8_t*)&d)[0]
 #elif __BIG_ENDIAN__
-
-#define STORE8_BE8(d) \
-	((uint8_t*)&d)[0]
-
-
-#define STORE16_BE8(d) \
-	((uint8_t*)&d)[1]
-
-#define STORE16_BE16(d) \
-	((uint8_t*)&d)[0], ((uint8_t*)&d)[1]
-
-
-#define STORE32_BE8(d) \
-	((uint8_t*)&d)[3]
-
-#define STORE32_BE16(d) \
-	((uint8_t*)&d)[2], ((uint8_t*)&d)[3]
-
-#define STORE32_BE32(d) \
-	((uint8_t*)&d)[0], ((uint8_t*)&d)[1], ((uint8_t*)&d)[2], ((uint8_t*)&d)[3]
-
-
-#define STORE64_BE8(d) \
-	((uint8_t*)&d)[7]
-
-#define STORE64_BE16(d) \
-	((uint8_t*)&d)[6], ((uint8_t*)&d)[7]
-
-#define STORE64_BE32(d) \
-	((uint8_t*)&d)[4], ((uint8_t*)&d)[5], ((uint8_t*)&d)[6], ((uint8_t*)&d)[7]
-
-#define STORE64_BE64(d) \
-	((uint8_t*)&d)[0], ((uint8_t*)&d)[1], ((uint8_t*)&d)[2], ((uint8_t*)&d)[3], \
-	((uint8_t*)&d)[4], ((uint8_t*)&d)[5], ((uint8_t*)&d)[6], ((uint8_t*)&d)[7]
-
+#define TAKE8_8(d)  ((uint8_t*)&d)[0]
+#define TAKE8_16(d) ((uint8_t*)&d)[1]
+#define TAKE8_32(d) ((uint8_t*)&d)[3]
+#define TAKE8_64(d) ((uint8_t*)&d)[7]
 #endif
 
 #ifndef msgpack_pack_inline_func
@@ -121,10 +49,10 @@
 do { \
 	if(d < (1<<7)) { \
 		/* fixnum */ \
-		msgpack_pack_append_buffer(x, &STORE8_BE8(d), 1); \
+		msgpack_pack_append_buffer(x, &TAKE8_8(d), 1); \
 	} else { \
 		/* unsigned 8 */ \
-		const unsigned char buf[2] = {0xcc, STORE8_BE8(d)}; \
+		unsigned char buf[2] = {0xcc, TAKE8_8(d)}; \
 		msgpack_pack_append_buffer(x, buf, 2); \
 	} \
 } while(0)
@@ -133,14 +61,15 @@ do { \
 do { \
 	if(d < (1<<7)) { \
 		/* fixnum */ \
-		msgpack_pack_append_buffer(x, &STORE16_BE8(d), 1); \
+		msgpack_pack_append_buffer(x, &TAKE8_16(d), 1); \
 	} else if(d < (1<<8)) { \
 		/* unsigned 8 */ \
-		const unsigned char buf[2] = {0xcc, STORE16_BE8(d)}; \
+		unsigned char buf[2] = {0xcc, TAKE8_16(d)}; \
 		msgpack_pack_append_buffer(x, buf, 2); \
 	} else { \
 		/* unsigned 16 */ \
-		const unsigned char buf[3] = {0xcd, STORE16_BE16(d)}; \
+		unsigned char buf[3]; \
+		buf[0] = 0xcd; _msgpack_store16(&buf[1], d); \
 		msgpack_pack_append_buffer(x, buf, 3); \
 	} \
 } while(0)
@@ -150,20 +79,22 @@ do { \
 	if(d < (1<<8)) { \
 		if(d < (1<<7)) { \
 			/* fixnum */ \
-			msgpack_pack_append_buffer(x, &STORE32_BE8(d), 1); \
+			msgpack_pack_append_buffer(x, &TAKE8_32(d), 1); \
 		} else { \
 			/* unsigned 8 */ \
-			const unsigned char buf[2] = {0xcc, STORE32_BE8(d)}; \
+			unsigned char buf[2] = {0xcc, TAKE8_32(d)}; \
 			msgpack_pack_append_buffer(x, buf, 2); \
 		} \
 	} else { \
 		if(d < (1<<16)) { \
 			/* unsigned 16 */ \
-			const unsigned char buf[3] = {0xcd, STORE32_BE16(d)}; \
+			unsigned char buf[3]; \
+			buf[0] = 0xcd; _msgpack_store16(&buf[1], d); \
 			msgpack_pack_append_buffer(x, buf, 3); \
 		} else { \
 			/* unsigned 32 */ \
-			const unsigned char buf[5] = {0xce, STORE32_BE32(d)}; \
+			unsigned char buf[5]; \
+			buf[0] = 0xce; _msgpack_store32(&buf[1], d); \
 			msgpack_pack_append_buffer(x, buf, 5); \
 		} \
 	} \
@@ -174,24 +105,27 @@ do { \
 	if(d < (1ULL<<8)) { \
 		if(d < (1<<7)) { \
 			/* fixnum */ \
-			msgpack_pack_append_buffer(x, &STORE64_BE8(d), 1); \
+			msgpack_pack_append_buffer(x, &TAKE8_64(d), 1); \
 		} else { \
 			/* unsigned 8 */ \
-			const unsigned char buf[2] = {0xcc, STORE64_BE8(d)}; \
+			unsigned char buf[2] = {0xcc, TAKE8_64(d)}; \
 			msgpack_pack_append_buffer(x, buf, 2); \
 		} \
 	} else { \
 		if(d < (1ULL<<16)) { \
 			/* signed 16 */ \
-			const unsigned char buf[3] = {0xcd, STORE64_BE16(d)}; \
+			unsigned char buf[3]; \
+			buf[0] = 0xcd; _msgpack_store16(&buf[1], d); \
 			msgpack_pack_append_buffer(x, buf, 3); \
 		} else if(d < (1ULL<<32)) { \
 			/* signed 32 */ \
-			const unsigned char buf[5] = {0xce, STORE64_BE32(d)}; \
+			unsigned char buf[5]; \
+			buf[0] = 0xce; _msgpack_store32(&buf[1], d); \
 			msgpack_pack_append_buffer(x, buf, 5); \
 		} else { \
 			/* signed 64 */ \
-			const unsigned char buf[9] = {0xcf, STORE64_BE64(d)}; \
+			unsigned char buf[9]; \
+			buf[0] = 0xcf; _msgpack_store64(&buf[1], d); \
 			msgpack_pack_append_buffer(x, buf, 9); \
 		} \
 	} \
@@ -201,11 +135,11 @@ do { \
 do { \
 	if(d < -(1<<5)) { \
 		/* signed 8 */ \
-		const unsigned char buf[2] = {0xd0, STORE8_BE8(d)}; \
+		unsigned char buf[2] = {0xd0, TAKE8_8(d)}; \
 		msgpack_pack_append_buffer(x, buf, 2); \
 	} else { \
 		/* fixnum */ \
-		msgpack_pack_append_buffer(x, &STORE8_BE8(d), 1); \
+		msgpack_pack_append_buffer(x, &TAKE8_8(d), 1); \
 	} \
 } while(0)
 
@@ -214,24 +148,26 @@ do { \
 	if(d < -(1<<5)) { \
 		if(d < -(1<<7)) { \
 			/* signed 16 */ \
-			const unsigned char buf[3] = {0xd1, STORE16_BE16(d)}; \
+			unsigned char buf[3]; \
+			buf[0] = 0xd1; _msgpack_store16(&buf[1], d); \
 			msgpack_pack_append_buffer(x, buf, 3); \
 		} else { \
 			/* signed 8 */ \
-			const unsigned char buf[2] = {0xd0, STORE16_BE8(d)}; \
+			unsigned char buf[2] = {0xd0, TAKE8_16(d)}; \
 			msgpack_pack_append_buffer(x, buf, 2); \
 		} \
 	} else if(d < (1<<7)) { \
 		/* fixnum */ \
-		msgpack_pack_append_buffer(x, &STORE16_BE8(d), 1); \
+		msgpack_pack_append_buffer(x, &TAKE8_16(d), 1); \
 	} else { \
 		if(d < (1<<8)) { \
 			/* unsigned 8 */ \
-			const unsigned char buf[2] = {0xcc, STORE16_BE8(d)}; \
+			unsigned char buf[2] = {0xcc, TAKE8_16(d)}; \
 			msgpack_pack_append_buffer(x, buf, 2); \
 		} else { \
 			/* unsigned 16 */ \
-			const unsigned char buf[3] = {0xcd, STORE16_BE16(d)}; \
+			unsigned char buf[3]; \
+			buf[0] = 0xcd; _msgpack_store16(&buf[1], d); \
 			msgpack_pack_append_buffer(x, buf, 3); \
 		} \
 	} \
@@ -242,32 +178,36 @@ do { \
 	if(d < -(1<<5)) { \
 		if(d < -(1<<15)) { \
 			/* signed 32 */ \
-			const unsigned char buf[5] = {0xd2, STORE32_BE32(d)}; \
+			unsigned char buf[5]; \
+			buf[0] = 0xd2; _msgpack_store32(&buf[1], d); \
 			msgpack_pack_append_buffer(x, buf, 5); \
 		} else if(d < -(1<<7)) { \
 			/* signed 16 */ \
-			const unsigned char buf[3] = {0xd1, STORE32_BE16(d)}; \
+			unsigned char buf[3]; \
+			buf[0] = 0xd1; _msgpack_store16(&buf[1], d); \
 			msgpack_pack_append_buffer(x, buf, 3); \
 		} else { \
 			/* signed 8 */ \
-			const unsigned char buf[2] = {0xd0, STORE32_BE8(d)}; \
+			unsigned char buf[2] = {0xd0, TAKE8_32(d)}; \
 			msgpack_pack_append_buffer(x, buf, 2); \
 		} \
 	} else if(d < (1<<7)) { \
 		/* fixnum */ \
-		msgpack_pack_append_buffer(x, &STORE32_BE8(d), 1); \
+		msgpack_pack_append_buffer(x, &TAKE8_32(d), 1); \
 	} else { \
 		if(d < (1<<8)) { \
 			/* unsigned 8 */ \
-			const unsigned char buf[2] = {0xcc, STORE32_BE8(d)}; \
+			unsigned char buf[2] = {0xcc, TAKE8_32(d)}; \
 			msgpack_pack_append_buffer(x, buf, 2); \
 		} else if(d < (1<<16)) { \
 			/* unsigned 16 */ \
-			const unsigned char buf[3] = {0xcd, STORE32_BE16(d)}; \
+			unsigned char buf[3]; \
+			buf[0] = 0xcd; _msgpack_store16(&buf[1], d); \
 			msgpack_pack_append_buffer(x, buf, 3); \
 		} else { \
 			/* unsigned 32 */ \
-			const unsigned char buf[5] = {0xce, STORE32_BE32(d)}; \
+			unsigned char buf[5]; \
+			buf[0] = 0xce; _msgpack_store32(&buf[1], d); \
 			msgpack_pack_append_buffer(x, buf, 5); \
 		} \
 	} \
@@ -279,46 +219,52 @@ do { \
 		if(d < -(1LL<<15)) { \
 			if(d < -(1LL<<31)) { \
 				/* signed 64 */ \
-				const unsigned char buf[9] = {0xd3, STORE64_BE64(d)}; \
+				unsigned char buf[9]; \
+				buf[0] = 0xd3; _msgpack_store64(&buf[1], d); \
 				msgpack_pack_append_buffer(x, buf, 9); \
 			} else { \
 				/* signed 32 */ \
-				const unsigned char buf[5] = {0xd2, STORE64_BE32(d)}; \
+				unsigned char buf[5]; \
+				buf[0] = 0xd2; _msgpack_store32(&buf[1], d); \
 				msgpack_pack_append_buffer(x, buf, 5); \
 			} \
 		} else { \
 			if(d < -(1<<7)) { \
 				/* signed 16 */ \
-				const unsigned char buf[3] = {0xd1, STORE64_BE16(d)}; \
+				unsigned char buf[3]; \
+				buf[0] = 0xd1; _msgpack_store16(&buf[1], d); \
 				msgpack_pack_append_buffer(x, buf, 3); \
 			} else { \
 				/* signed 8 */ \
-				const unsigned char buf[2] = {0xd0, STORE64_BE8(d)}; \
+				unsigned char buf[2] = {0xd0, TAKE8_64(d)}; \
 				msgpack_pack_append_buffer(x, buf, 2); \
 			} \
 		} \
 	} else if(d < (1<<7)) { \
 		/* fixnum */ \
-		msgpack_pack_append_buffer(x, &STORE64_BE8(d), 1); \
+		msgpack_pack_append_buffer(x, &TAKE8_64(d), 1); \
 	} else { \
 		if(d < (1LL<<16)) { \
 			if(d < (1<<8)) { \
 				/* unsigned 8 */ \
-				const unsigned char buf[2] = {0xcc, STORE64_BE8(d)}; \
+				unsigned char buf[2] = {0xcc, TAKE8_64(d)}; \
 				msgpack_pack_append_buffer(x, buf, 2); \
 			} else { \
 				/* unsigned 16 */ \
-				const unsigned char buf[3] = {0xcd, STORE64_BE16(d)}; \
+				unsigned char buf[3]; \
+				buf[0] = 0xcd; _msgpack_store16(&buf[1], d); \
 				msgpack_pack_append_buffer(x, buf, 3); \
 			} \
 		} else { \
 			if(d < (1LL<<32)) { \
 				/* unsigned 32 */ \
-				const unsigned char buf[5] = {0xce, STORE64_BE32(d)}; \
+				unsigned char buf[5]; \
+				buf[0] = 0xce; _msgpack_store32(&buf[1], d); \
 				msgpack_pack_append_buffer(x, buf, 5); \
 			} else { \
 				/* unsigned 64 */ \
-				const unsigned char buf[9] = {0xcf, STORE64_BE64(d)}; \
+				unsigned char buf[9]; \
+				buf[0] = 0xcf; _msgpack_store64(&buf[1], d); \
 				msgpack_pack_append_buffer(x, buf, 9); \
 			} \
 		} \
@@ -330,49 +276,55 @@ do { \
 
 msgpack_pack_inline_func_fastint(_uint8)(msgpack_pack_user x, uint8_t d)
 {
-	const unsigned char buf[2] = {0xcc, STORE8_BE8(d)};
+	unsigned char buf[2] = {0xcc, TAKE8_8(d)};
 	msgpack_pack_append_buffer(x, buf, 2);
 }
 
 msgpack_pack_inline_func_fastint(_uint16)(msgpack_pack_user x, uint16_t d)
 {
-	const unsigned char buf[3] = {0xcd, STORE16_BE16(d)};
+	unsigned char buf[3];
+	buf[0] = 0xcd; _msgpack_store16(&buf[1], d);
 	msgpack_pack_append_buffer(x, buf, 3);
 }
 
 msgpack_pack_inline_func_fastint(_uint32)(msgpack_pack_user x, uint32_t d)
 {
-	const unsigned char buf[5] = {0xce, STORE32_BE32(d)};
+	unsigned char buf[5];
+	buf[0] = 0xce; _msgpack_store32(&buf[1], d);
 	msgpack_pack_append_buffer(x, buf, 5);
 }
 
 msgpack_pack_inline_func_fastint(_uint64)(msgpack_pack_user x, uint64_t d)
 {
-	const unsigned char buf[9] = {0xcf, STORE64_BE64(d)};
+	unsigned char buf[9];
+	buf[0] = 0xcf; _msgpack_store64(&buf[1], d);
 	msgpack_pack_append_buffer(x, buf, 9);
 }
 
 msgpack_pack_inline_func_fastint(_int8)(msgpack_pack_user x, int8_t d)
 {
-	const unsigned char buf[2] = {0xd0, STORE8_BE8(d)};
+	unsigned char buf[2] = {0xd0, TAKE8_8(d)};
 	msgpack_pack_append_buffer(x, buf, 2);
 }
 
 msgpack_pack_inline_func_fastint(_int16)(msgpack_pack_user x, int16_t d)
 {
-	const unsigned char buf[3] = {0xd1, STORE16_BE16(d)};
+	unsigned char buf[3];
+	buf[0] = 0xd1; _msgpack_store16(&buf[1], d);
 	msgpack_pack_append_buffer(x, buf, 3);
 }
 
 msgpack_pack_inline_func_fastint(_int32)(msgpack_pack_user x, int32_t d)
 {
-	const unsigned char buf[5] = {0xd2, STORE32_BE32(d)};
+	unsigned char buf[5];
+	buf[0] = 0xd2; _msgpack_store32(&buf[1], d);
 	msgpack_pack_append_buffer(x, buf, 5);
 }
 
 msgpack_pack_inline_func_fastint(_int64)(msgpack_pack_user x, int64_t d)
 {
-	const unsigned char buf[9] = {0xd3, STORE64_BE64(d)};
+	unsigned char buf[9];
+	buf[0] = 0xd3; _msgpack_store64(&buf[1], d);
 	msgpack_pack_append_buffer(x, buf, 9);
 }
 
@@ -602,17 +554,19 @@ if(sizeof(unsigned long long) == 2) {
 
 msgpack_pack_inline_func(_float)(msgpack_pack_user x, float d)
 {
-	union { char buf[4]; uint32_t num; } f;
-	*((float*)&f.buf) = d;  // FIXME
-	const unsigned char buf[5] = {0xca, STORE32_BE32(f.num)};
+	union { float f; uint32_t i; } mem;
+	mem.f = d;
+	unsigned char buf[5];
+	buf[0] = 0xca; _msgpack_store32(&buf[1], mem.i);
 	msgpack_pack_append_buffer(x, buf, 5);
 }
 
 msgpack_pack_inline_func(_double)(msgpack_pack_user x, double d)
 {
-	union { char buf[8]; uint64_t num; } f;
-	*((double*)&f.buf) = d;  // FIXME
-	const unsigned char buf[9] = {0xcb, STORE64_BE64(f.num)};
+	union { double f; uint64_t i; } mem;
+	mem.f = d;
+	unsigned char buf[9];
+	buf[0] = 0xcb; _msgpack_store64(&buf[1], mem.i);
 	msgpack_pack_append_buffer(x, buf, 9);
 }
 
@@ -655,12 +609,12 @@ msgpack_pack_inline_func(_array)(msgpack_pack_user x, unsigned int n)
 		unsigned char d = 0x90 | n;
 		msgpack_pack_append_buffer(x, &d, 1);
 	} else if(n < 65536) {
-		uint16_t d = (uint16_t)n;
-		unsigned char buf[3] = {0xdc, STORE16_BE16(d)};
+		unsigned char buf[3];
+		buf[0] = 0xdc; _msgpack_store16(&buf[1], n);
 		msgpack_pack_append_buffer(x, buf, 3);
 	} else {
-		uint32_t d = (uint32_t)n;
-		unsigned char buf[5] = {0xdd, STORE32_BE32(d)};
+		unsigned char buf[5];
+		buf[0] = 0xdd; _msgpack_store32(&buf[1], n);
 		msgpack_pack_append_buffer(x, buf, 5);
 	}
 }
@@ -674,14 +628,14 @@ msgpack_pack_inline_func(_map)(msgpack_pack_user x, unsigned int n)
 {
 	if(n < 16) {
 		unsigned char d = 0x80 | n;
-		msgpack_pack_append_buffer(x, &STORE8_BE8(d), 1);
+		msgpack_pack_append_buffer(x, &TAKE8_8(d), 1);
 	} else if(n < 65536) {
-		uint16_t d = (uint16_t)n;
-		unsigned char buf[3] = {0xde, STORE16_BE16(d)};
+		unsigned char buf[3];
+		buf[0] = 0xde; _msgpack_store16(&buf[1], n);
 		msgpack_pack_append_buffer(x, buf, 3);
 	} else {
-		uint32_t d = (uint32_t)n;
-		unsigned char buf[5] = {0xdf, STORE32_BE32(d)};
+		unsigned char buf[5];
+		buf[0] = 0xdf; _msgpack_store32(&buf[1], n);
 		msgpack_pack_append_buffer(x, buf, 5);
 	}
 }
@@ -695,14 +649,14 @@ msgpack_pack_inline_func(_raw)(msgpack_pack_user x, size_t l)
 {
 	if(l < 32) {
 		unsigned char d = 0xa0 | l;
-		msgpack_pack_append_buffer(x, &STORE8_BE8(d), 1);
+		msgpack_pack_append_buffer(x, &TAKE8_8(d), 1);
 	} else if(l < 65536) {
-		uint16_t d = (uint16_t)l;
-		unsigned char buf[3] = {0xda, STORE16_BE16(d)};
+		unsigned char buf[3];
+		buf[0] = 0xda; _msgpack_store16(&buf[1], l);
 		msgpack_pack_append_buffer(x, buf, 3);
 	} else {
-		uint32_t d = (uint32_t)l;
-		unsigned char buf[5] = {0xdb, STORE32_BE32(d)};
+		unsigned char buf[5];
+		buf[0] = 0xdb; _msgpack_store32(&buf[1], l);
 		msgpack_pack_append_buffer(x, buf, 5);
 	}
 }
@@ -716,19 +670,10 @@ msgpack_pack_inline_func(_raw_body)(msgpack_pack_user x, const void* b, size_t l
 #undef msgpack_pack_user
 #undef msgpack_pack_append_buffer
 
-#undef STORE8_BE8
-
-#undef STORE16_BE8
-#undef STORE16_BE16
-
-#undef STORE32_BE8
-#undef STORE32_BE16
-#undef STORE32_BE32
-
-#undef STORE64_BE8
-#undef STORE64_BE16
-#undef STORE64_BE32
-#undef STORE64_BE64
+#undef TAKE8_8
+#undef TAKE8_16
+#undef TAKE8_32
+#undef TAKE8_64
 
 #undef msgpack_pack_real_uint8
 #undef msgpack_pack_real_uint16
